@@ -10,24 +10,23 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: guienv.ml,v 1.6 2004/09/23 06:56:23 weis Exp $ *)
+(* $Id: guienv.ml,v 1.6.2.1 2010/05/13 13:36:09 furuse Exp $ *)
 
-open Gdk;;
-open GDraw;;
-open GMain;;
+open Gdk
+open GMain
 
-GtkMain.Main.init ();;
+let () = ignore @@ GtkMain.Main.init ()
 
 let window = GWindow.window ~title: "gifanim"
-    ~auto_shrink: true 
+(*    ~auto_shrink: true  *)
     ~allow_shrink: true 
-    ~allow_grow: true ();;
+    ~allow_grow: true ()
 
-window#connect#destroy ~callback:Main.quit;;
+let () = ignore @@ window#connect#destroy ~callback:Main.quit
 
-let visual = window#misc#visual;;
-let quick_color_create = Truecolor.color_creator visual;;
-let quick_color_parser = Truecolor.color_parser visual;;
+let visual = window#misc#visual
+let quick_color_create = Truecolor.color_creator visual
+let quick_color_parser = Truecolor.color_parser visual
 
-let colormap = Gdk.Color.get_system_colormap ();;
+let colormap = Gdk.Color.get_system_colormap ()
 
