@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: mylazy.ml,v 1.1 2002/04/09 11:00:09 furuse Exp $ *)
+(* $Id: mylazy.ml,v 1.2 2004/09/23 07:20:20 weis Exp $ *)
 
 (* Module [Lazy]: deferred computations *)
 
@@ -19,11 +19,11 @@ type 'a status =
   | Delayed of (unit -> 'a)
   | Value of 'a
   | Exception of exn
-;;
 
-type 'a t = 'a status ref;;
 
-exception Undefined;;
+type 'a t = 'a status ref
+
+exception Undefined
 
 let make f = ref (Delayed f)
 let make_val v = ref (Value v)
@@ -36,4 +36,4 @@ let force l =
       l := Exception Undefined;
       try let v = f () in l := Value v; v
       with e -> l := Exception e; raise e
-;;
+
