@@ -10,11 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: scandir.ml,v 1.9 2004/09/23 07:20:20 weis Exp $ *)
+(* $Id: scandir.ml,v 1.10 2008/06/16 22:35:42 furuse Exp $ *)
 
-open Unix;;
+open Unix
 
-let ignore_dirs = ref [".xvpics"];;
+let ignore_dirs = ref [".xvpics"; ".livpics"; "0000HEADER"; "0000HEADERS"]
 
 let scan_dir f fn =
   let scanned = ref [] in
@@ -31,7 +31,7 @@ let scan_dir f fn =
         | e -> prerr_endline ("readdir: " ^ Printexc.to_string e)
       end;
       closedir dh;
-      let files = Sort.list (>) !files in
+      let files = List.sort (fun x y -> compare y x) !files in
       let subdirs = ref [] in
       let treat fn' =
         if not (fn' = ".." || fn' = ".") then begin
@@ -52,4 +52,4 @@ let scan_dir f fn =
       List.iter sub (List.rev !subdirs)
     | S_DIR -> ()
     | _ -> f fn in
-  sub fn;;
+  sub fn
