@@ -11,22 +11,28 @@
 /*  Distributed only by permission.                                    */
 /*                                                                     */
 /***********************************************************************/
-#include <config.h>
+
+#include "../config/config.h"
+
+#ifdef HAS_GIF
+
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
 #include <caml/fail.h>
 
+#include "../include/oversized.h"
+
 #include <stdio.h>
-
-#define gif_not_supported() \
-  failwith( "gif is not supported" ); \
-  return(Val_unit)
-
-#if HAVE_GIF
+#include <string.h>
 
 #include <gif_lib.h>
 
+// GIFLIB_MAJOR is only defined in libgif >= 4.2.0
+#if !defined(GIFLIB_MAJOR)
+#  define GIFLIB_MAJOR 4
+#endif
+
 value Val_GifColorType( GifColorType *color )
 {
   CAMLparam0();
@@ -37,7 +43,7 @@ value Val_GifColorType( GifColorType *color )
   r[0] = Val_int( color->Red );
   r[1] = Val_int( color->Green );
   r[2] = Val_int( color->Blue );
-  res = alloc_tuple(3);
+  res = alloc_small(3,0);
   for(i=0; i<3; i++) Field(res, i) = r[i];
 #ifdef DEBUG_GIF
 fprintf(stderr, "Color(%d,%d,%d)\n", color->Red, color->Green, color->Blue);
@@ -60,7 +66,7 @@ value Val_ColorMapObject( ColorMapObject *colorMap )
 
     cmap = alloc_tuple(colorMap->ColorCount);
     for(i= 0; i< colorMap->ColorCount; i++){
-      modify(&Field(cmap,i), Val_GifColorType( &colorMap->Colors[i] ));
+        Store_field(cmap,i, Val_GifColorType( &colorMap->Colors[i] ));
     }
   } else {
 #ifdef DEBUG_GIF
@@ -107,7 +113,7 @@ fflush(stderr);
   r[3] = Val_int( imageDesc->Height );
   r[4] = Val_int( imageDesc->Interlace );
   r[5] = Val_ColorMapObject( imageDesc->ColorMap );
-  res = alloc_tuple(6);
+  res = alloc_small(6,0);
   for(i=0; i<6; i++) Field(res, i) = r[i];
   CAMLreturn(res);
 }
@@ -125,7 +131,7 @@ value Val_ScreenInfo( GifFileType *GifFile )
   r[2] = Val_int(GifFile->SColorResolution);
   r[3] = Val_int(GifFile->SBackGroundColor);
   r[4] = Val_ColorMapObject(GifFile->SColorMap);
-  res = alloc_tuple(5);
+  res = alloc_small(5,0);
   for(i=0; i<5; i++) Field(res, i) = r[i];
 
   CAMLreturn(res);
@@ -140,19 +146,25 @@ value dGifOpenFileName( value name )
   GifFileType *GifFile;
   int i;
 
-  if((GifFile = DGifOpenFileName( String_val(name) )) == NULL){
+#if (GIFLIB_MAJOR <= 4)
+    GifFile = DGifOpenFileName( String_val(name) );
+#else
+    GifFile = DGifOpenFileName( String_val(name), NULL);
+#endif
+
+  if(GifFile == NULL){
     failwith("DGifOpenFileName");
   }
 
   r[0] = Val_ScreenInfo( GifFile );
   r[1] = (value) GifFile;
-  res = alloc_tuple(2);
+  res = alloc_small(2,0);
   for(i=0; i<2; i++) Field(res, i) = r[i];
 
   CAMLreturn(res);
 } 
 
-value dGifCloseFile( value hdl )
+void dGifCloseFile( value hdl )
 {
   CAMLparam1(hdl);
 
@@ -161,8 +173,12 @@ value dGifCloseFile( value hdl )
      segmentation faults */
   ((GifFileType *)hdl)->Image.ColorMap = NULL; 
 
-  DGifCloseFile( (GifFileType *) hdl );
-  CAMLreturn(Val_unit);
+#if (GIFLIB_MAJOR <= 4)
+  DGifCloseFile( (GifFileType *) hdl);
+#else
+  DGifCloseFile( (GifFileType *) hdl, NULL );
+#endif
+  CAMLreturn0;
 }
 
 value dGifGetRecordType( value hdl )
@@ -193,11 +209,14 @@ value dGifGetLine( value hdl )
 
   GifFileType *GifFile = (GifFileType*) hdl;
 
+  if( oversized( GifFile->Image.Width, sizeof(GifPixelType) ) ){
+    failwith_oversized("gif");
+  }
   buf = alloc_string( GifFile->Image.Width * sizeof(GifPixelType) ); 
 
-  if( DGifGetLine(GifFile, String_val(buf), GifFile->Image.Width ) 
+  if( DGifGetLine(GifFile, (unsigned char*)String_val(buf), GifFile->Image.Width ) 
       == GIF_ERROR ){
-    PrintGifError ();
+    // PrintGifError ();
     failwith("DGifGetLine");
   }
   CAMLreturn(buf);
@@ -228,22 +247,35 @@ value dGifGetExtension( value hdl )
     exts= newres;
     DGifGetExtensionNext(GifFile, &extData);
   }
-  res = alloc_tuple(2);
+  res = alloc_small(2,0);
   Field(res,0) = Val_int(func);
   Field(res,1) = exts;
 
   CAMLreturn(res);
 }
-     
-#else
 
-#define nodef(f) value f(){gif_not_supported();}
+#else // HAS_GIF
 
-nodef(dGifOpenFileName)
-nodef(dGifCloseFile)
-nodef(dGifGetRecordType)
-nodef(dGifGetImageDesc)
-nodef(dGifGetLine)
-nodef(dGifGetExtensions)
-nodef(dGifGetExtension)
-#endif
+#include <caml/mlvalues.h>
+#include <caml/alloc.h>
+#include <caml/memory.h>
+#include <caml/fail.h>
+
+value Val_GifColorType(){ failwith("unsupported"); }
+value Val_ColorMapObject(){ failwith("unsupported"); }
+value Val_GifImageDesc(){ failwith("unsupported"); }
+value Val_ScreenInfo(){ failwith("unsupported"); }
+value dGifOpenFileName(){ failwith("unsupported"); }
+value dGifCloseFile(){ failwith("unsupported"); }
+value dGifGetRecordType(){ failwith("unsupported"); }
+value dGifGetImageDesc(){ failwith("unsupported"); }
+value dGifGetLine(){ failwith("unsupported"); }
+value dGifGetExtension(){ failwith("unsupported"); }
+value eGifOpenFileName(){ failwith("unsupported"); }
+value eGifCloseFile(){ failwith("unsupported"); }
+value eGifPutScreenDesc(){ failwith("unsupported"); }
+value eGifPutImageDesc(){ failwith("unsupported"); }
+value eGifPutLine(){ failwith("unsupported"); }
+value eGifPutExtension(){ failwith("unsupported"); }
+
+#endif // HAS_GIF
