@@ -11,18 +11,26 @@
 /*  Distributed only by permission.                                    */
 /*                                                                     */
 /***********************************************************************/
-#include <config.h>
+
+#include "../config/config.h"
+
+#ifdef HAS_GIF
+
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
 #include <caml/fail.h>
 
 #include <stdio.h>
-
-#if HAVE_GIF
+#include <string.h>
 
 #include <gif_lib.h>
 
+// GIFLIB_MAJOR is only defined in libgif >= 4.2.0
+#if !defined(GIFLIB_MAJOR)
+#  define GIFLIB_MAJOR 4
+#endif
+
 int list_length( value list )
 {
   CAMLparam1(list);
@@ -31,13 +39,12 @@ int list_length( value list )
   for(l = list; l != Val_int(0); l = Field(l,1)){
     length ++;
   }
-  return length;
+  CAMLreturn(length);
 }
 
 ColorMapObject *ColorMapObject_val( value cmap )
 {
-  CAMLparam1(cmap);
-
+  // no caml allocation inside.
   int len;
   int i;
   ColorMapObject *cmapobj;
@@ -50,7 +57,12 @@ ColorMapObject *ColorMapObject_val( value cmap )
 fprintf(stderr, "Creating map with length = %d ...\n", len);
 fflush(stderr);
 */
-  cmapobj = MakeMapObject( len, NULL );
+#if (GIFLIB_MAJOR == 4)
+    cmapobj = MakeMapObject( len, NULL );
+#else
+    cmapobj = GifMakeMapObject( len, NULL );
+#endif
+
   for(i=0; i< len; i++){
     cmapobj->Colors[i].Red   = Int_val(Field(Field(cmap,i),0));
     cmapobj->Colors[i].Green = Int_val(Field(Field(cmap,i),1));
@@ -59,27 +71,6 @@ fflush(stderr);
   return cmapobj; 
 }
 
-/*
-ColorMapObject *ColorMapObject_val( value cmap )
-{
-  int len;
-  int i;
-  ColorMapObject *cmapobj;
-  value l;
-
-  len = list_length(cmap);
-fprintf(stderr, "Creating map with length = %d ...\n", len);
-fflush(stderr);
-  cmapobj = MakeMapObject( len, NULL );
-  for(i=0, l= cmap; i< len; i++, l = Field(l,1)){
-    cmapobj->Colors[i].Red   = Int_val(Field(Field(l,0),0));
-    cmapobj->Colors[i].Green = Int_val(Field(Field(l,0),1));
-    cmapobj->Colors[i].Blue  = Int_val(Field(Field(l,0),2));
-  }
-  return cmapobj; 
-}
-*/
-
 value eGifOpenFileName( name )
      value name;
 {
@@ -87,7 +78,13 @@ value eGifOpenFileName( name )
 
   GifFileType *GifFileOut;
 
-  if ((GifFileOut = EGifOpenFileName( String_val( name ), 0) )== NULL) {
+#if (GIFLIB_MAJOR <= 4)
+    GifFileOut = EGifOpenFileName( String_val( name ), 0 );
+#else
+    GifFileOut = EGifOpenFileName( String_val( name ), 0, NULL );
+#endif
+
+  if (GifFileOut == NULL) {
     failwith("EGifOpenFileName");
   }
   /* gcc -fwritable-strings is required to compile libungif */
@@ -98,7 +95,7 @@ value eGifOpenFileName( name )
   CAMLreturn((value)GifFileOut);
 }
 
-value eGifCloseFile( value hdl )
+void eGifCloseFile( value hdl )
 {
   CAMLparam1(hdl);
 
@@ -107,11 +104,15 @@ value eGifCloseFile( value hdl )
      segmentation faults */
   ((GifFileType *)hdl)->Image.ColorMap = NULL; 
 
+#if (GIFLIB_MAJOR <= 4)
   EGifCloseFile( (GifFileType *) hdl );
-  CAMLreturn(Val_unit);
+#else
+  EGifCloseFile( (GifFileType *) hdl, NULL );
+#endif
+  CAMLreturn0;
 }
 
-value eGifPutScreenDesc( value oc, value sdesc )
+void eGifPutScreenDesc( value oc, value sdesc )
 {
   CAMLparam2(oc,sdesc);
 
@@ -124,10 +125,10 @@ value eGifPutScreenDesc( value oc, value sdesc )
 			 ColorMapObject_val( Field(sdesc, 4) )) == GIF_ERROR){
     failwith("EGifPutScreenDesc");
   }
-  CAMLreturn(Val_unit);
+  CAMLreturn0;
 }
 
-value eGifPutImageDesc( value oc, value idesc )
+void eGifPutImageDesc( value oc, value idesc )
 {
   CAMLparam2(oc,idesc);
 
@@ -141,24 +142,24 @@ value eGifPutImageDesc( value oc, value idesc )
 			ColorMapObject_val( Field(idesc, 5) )) == GIF_ERROR){
     failwith("EGifPutImageDesc");
   }
-  CAMLreturn(Val_unit);
+  CAMLreturn0;
 }
 
-value eGifPutLine( value oc, value buf )
+void eGifPutLine( value oc, value buf )
 {
   CAMLparam2(oc,buf);
 
   GifFileType *GifFileOut = (GifFileType*) oc;
 
-  if ( EGifPutLine(GifFileOut, String_val(buf), GifFileOut->Image.Width) 
+  if ( EGifPutLine(GifFileOut, (unsigned char*)String_val(buf), GifFileOut->Image.Width) 
        == GIF_ERROR ){
-    PrintGifError ();
+    // PrintGifError ();
     failwith("EGifPutLine");
   }
-  CAMLreturn(Val_unit);
+  CAMLreturn0;
 }
 
-value eGifPutExtension( value oc, value ext )
+void eGifPutExtension( value oc, value ext )
 {
   CAMLparam2(oc,ext);
   CAMLlocal1(l);
@@ -195,20 +196,7 @@ value eGifPutExtension( value oc, value ext )
     free(extension);
     failwith("EGifPutExtension");
   }
-  CAMLreturn(Val_unit);
+  CAMLreturn0;
 }
 
-#else
-
-#define nodef(f) value f(){failwith( "gif is not supported" ); \
-  return(Val_unit);}
-
-nodef(write_gif_file)
-nodef(eGifOpenFileName)
-nodef(eGifCloseFile)
-nodef(eGifPutScreenDesc)
-nodef(eGifPutImageDesc)
-nodef(eGifPutLine)
-nodef(eGifPutExtension)
-
-#endif
+#endif // HAS_GIF
