@@ -2,7 +2,7 @@
 /*                                                                     */
 /*                           Objective Caml                            */
 /*                                                                     */
-/*            Franois Pessaux, projet Cristal, INRIA Rocquencourt     */
+/*            Fran輟is Pessaux, projet Cristal, INRIA Rocquencourt     */
 /*            Pierre Weis, projet Cristal, INRIA Rocquencourt          */
 /*            Jun Furuse, projet Cristal, INRIA Rocquencourt           */
 /*                                                                     */
@@ -11,17 +11,19 @@
 /*  Distributed only by permission.                                    */
 /*                                                                     */
 /***********************************************************************/
-#include <config.h>
+
+/* $Id: jpegread.c,v 1.5 2009/10/16 16:08:33 weis Exp $ */
+
+#include "../config/config.h"
+
+#ifdef HAS_JPEG
+
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
 #include <caml/fail.h>
 
-#define jpeg_not_supported() \
- failwith( "jpeg is not supported" ); \
- return (Val_unit)
-
-#if HAVE_JPEG
+#include "../include/oversized.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -43,6 +45,12 @@
 
 #include <setjmp.h>
 
+#ifdef DEBUG_JPEG
+#  define DEBUGF(...) do{fprintf(stderr, __VA_ARGS__); fflush(stderr); }while(0)
+#else
+#  define DEBUGF(...) do{/*do nothing*/}while(0)
+#endif
+
 char jpg_error_message[JMSG_LENGTH_MAX];
  
 struct my_error_mgr {
@@ -54,172 +62,20 @@ struct my_error_mgr {
 typedef struct my_error_mgr * my_error_ptr;
 
 void
-my_error_exit (j_common_ptr cinfo)
+my_error_exit (j_common_ptr cinfop)
 {
   /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
-  my_error_ptr myerr = (my_error_ptr) cinfo->err;
+  my_error_ptr myerr = (my_error_ptr) cinfop->err;
 
   /* Always display the message. */
   /* We could postpone this until after returning, if we chose. */
-  fprintf(stderr,"setting message\n");
-  (*cinfo->err->format_message) (cinfo, jpg_error_message );
+  fprintf(stderr,"setting libjpeg error message: \"%s\"\n", jpg_error_message);
+  (*cinfop->err->format_message) (cinfop, jpg_error_message );
 
   /* Return control to the setjmp point */
   longjmp(myerr->setjmp_buffer, 1);
 }
 
-GLOBAL(value)
-read_JPEG_file (value name)
-{
-  CAMLparam1(name);
-  CAMLlocal1(res);
-
-  char *filename;
-  /* This struct contains the JPEG decompression parameters and pointers to
-   * working space (which is allocated as needed by the JPEG library).
-   */
-  struct jpeg_decompress_struct cinfo;
-  /* We use our private extension JPEG error handler.
-   * Note that this struct must live as long as the main JPEG parameter
-   * struct, to avoid dangling-pointer problems.
-   */
-  struct my_error_mgr jerr;
-  /* More stuff */
-  FILE * infile;		/* source file */
-  JSAMPARRAY buffer;		/* Output row buffer */
-  int row_stride;		/* physical row width in output buffer */
-  int i;
-
-  filename= String_val( name );
-
-  /* In this example we want to open the input file before doing anything else,
-   * so that the setjmp() error recovery below can assume the file is open.
-   * VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
-   * requires it in order to read binary files.
-   */
-
-  if ((infile = fopen(filename, "rb")) == NULL) {
-    failwith("failed to open jpeg file");
-  }
-
-  /* Step 1: allocate and initialize JPEG decompression object */
-
-  /* We set up the normal JPEG error routines, then override error_exit. */
-  cinfo.err = jpeg_std_error(&jerr.pub);
-  jerr.pub.error_exit = my_error_exit;
-
-  /* Establish the setjmp return context for my_error_exit to use. */
-  if (setjmp(jerr.setjmp_buffer)) {
-    /* If we get here, the JPEG code has signaled an error.
-     * We need to clean up the JPEG object, close the input file, and return.
-     */
-    fprintf(stderr, "Exiting...");
-    jpeg_destroy_decompress(&cinfo);
-    fclose(infile);
-    exit(-1);
-    failwith(jpg_error_message);
-  }
-  /* Now we can initialize the JPEG decompression object. */
-  jpeg_create_decompress(&cinfo);
-
-  /* Step 2: specify data source (eg, a file) */
-
-  jpeg_stdio_src(&cinfo, infile);
-
-  /* Step 3: read file parameters with jpeg_read_header() */
-
-  (void) jpeg_read_header(&cinfo, TRUE);
-
-  /* We can ignore the return value from jpeg_read_header since
-   *   (a) suspension is not possible with the stdio data source, and
-   *   (b) we passed TRUE to reject a tables-only JPEG file as an error.
-   * See libjpeg.doc for more info.
-   */
-
-  /* Step 4: set parameters for decompression */
-
-  /* In this example, we don't need to change any of the defaults set by
-   * jpeg_read_header(), so we do nothing here.
-   */
-
-  cinfo.out_color_space = JCS_RGB;
-
-  /* Step 5: Start decompressor */
-
-  (void) jpeg_start_decompress(&cinfo);
-  /* We can ignore the return value since suspension is not possible
-   * with the stdio data source.
-   */
-
-  /* We may need to do some setup of our own at this point before reading
-   * the data.  After jpeg_start_decompress() we have the correct scaled
-   * output image dimensions available, as well as the output colormap
-   * if we asked for color quantization.
-   * In this example, we need to make an output work buffer of the right size.
-   */ 
-  /* JSAMPLEs per row in output buffer */
-
-  row_stride = cinfo.output_width * cinfo.output_components;
-
-  /* Make a one-row-high sample array that will go away when done with image */
-  buffer = (*cinfo.mem->alloc_sarray)
-		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 
-		 cinfo.output_height );
-
-  /* Step 6: while (scan lines remain to be read) */
-  /*           jpeg_read_scanlines(...); */
-
-  /* Here we use the library's state variable cinfo.output_scanline as the
-   * loop counter, so that we don't have to keep track ourselves.
-   */
-  while (cinfo.output_scanline < cinfo.output_height) {
-    /* jpeg_read_scanlines expects an array of pointers to scanlines.
-     * Here the array is only one element long, but you could ask for
-     * more than one scanline at a time if that's more convenient.
-     */
-    jpeg_read_scanlines(&cinfo, buffer + cinfo.output_scanline, 1); 
-  }
-
-  {
-    CAMLlocalN(r,3);
-    r[0] = Val_int(cinfo.output_width);
-    r[1] = Val_int(cinfo.output_height);
-    r[2] = alloc_string ( row_stride * cinfo.output_height );
-    for(i=0; i<cinfo.output_height; i++){
-      memcpy( String_val(r[2]) + i * row_stride, 
-	       buffer[i], row_stride);
-    }
-    res = alloc_tuple(3);
-    for(i=0; i<3; i++) Field(res, i) = r[i];
-  }
-
-  /* Step 7: Finish decompression */
-
-  (void) jpeg_finish_decompress(&cinfo);
-  /* We can ignore the return value since suspension is not possible
-   * with the stdio data source.
-   */
-
-  /* Step 8: Release JPEG decompression object */
-
-  /* This is an important step since it will release a good deal of memory. */
-  jpeg_destroy_decompress(&cinfo);
-
-  /* After finish_decompress, we can close the input file.
-   * Here we postpone it until after no more JPEG errors are possible,
-   * so as to simplify the setjmp error logic above.  (Actually, I don't
-   * think that jpeg_destroy can do an error exit, but why assume anything...)
-   */
-  fclose(infile);
-
-  /* At this point you may want to check to see whether any corrupt-data
-   * warnings occurred (test whether jerr.pub.num_warnings is nonzero).
-   */
-
-  /* And we're done! */
-  CAMLreturn(res);
-}
-
 value jpeg_set_scale_denom( jpegh, denom )
      value jpegh;
      value denom;
@@ -228,15 +84,53 @@ value jpeg_set_scale_denom( jpegh, denom )
   struct jpeg_decompress_struct *cinfop;
 
   cinfop = (struct jpeg_decompress_struct *) Field ( jpegh, 0 );
+  cinfop->scale_num = 1;
   cinfop->scale_denom = Int_val( denom );
   CAMLreturn(Val_unit);
 }
 
+value caml_val_jpeg_marker( jpeg_saved_marker_ptr p )
+{
+    CAMLparam0();
+    CAMLlocal2(res, tmp);
+
+    tmp = caml_alloc_string(p->data_length);
+    memcpy( String_val(tmp), p->data, p->data_length);
+
+    res = alloc_small(2,0);
+    Field(res, 0) = Val_int(p->marker);
+    Field(res, 1) = tmp;
+    // fprintf(stderr, "mark %x %d\n", p->marker, p->data_length);
+
+    CAMLreturn(res);
+}
+
+value caml_val_jpeg_rev_markers( jpeg_saved_marker_ptr q )
+{
+    CAMLparam0();
+    CAMLlocal3(res, tmp, hd);
+    jpeg_saved_marker_ptr p = q;
+    res = Val_int(0); // null
+    while(p){
+        hd = caml_val_jpeg_marker(p);
+
+        // This cannot come before the call of caml_val_jpeg_marker!
+        // New allocation with non-initialized blocks crashes GC!
+        tmp = alloc_small(2,0); 
+        Field(tmp, 0) = hd;
+        Field(tmp, 1) = res;
+        res = tmp;
+        p = p->next;
+    }
+    CAMLreturn(res);
+}
+
 value open_jpeg_file_for_read( name )
      value name;
 {
   CAMLparam1(name);
   CAMLlocal1(res);
+  CAMLlocalN(r,4);
 
   char *filename;
   /* This struct contains the JPEG decompression parameters and pointers to
@@ -290,21 +184,28 @@ value open_jpeg_file_for_read( name )
 
   jpeg_stdio_src(cinfop, infile);
 
+  /* EXIF and other markers */
+
+  jpeg_save_markers(cinfop, JPEG_COM, 0xFFFF /* all */);
+  for(i=0; i<16; i++)
+      jpeg_save_markers(cinfop, JPEG_APP0+i, 0xFFFF /* all */);
+
   /* Step 3: read file parameters with jpeg_read_header() */
 
   (void) jpeg_read_header(cinfop, TRUE);
 
-  { 
-    CAMLlocalN(r,3);
-    r[0] = Val_int(cinfop->image_width);
-    r[1] = Val_int(cinfop->image_height);
-    r[2] = alloc_tuple(3);
-    Field(r[2], 0) = (value)cinfop;
-    Field(r[2], 1) = (value)infile;
-    Field(r[2], 2) = (value)jerrp;
-    res = alloc_tuple(3);
-    for(i=0; i<3; i++) Field(res, i) = r[i];
-  }
+  r[0] = Val_int(cinfop->image_width);
+  r[1] = Val_int(cinfop->image_height);
+  r[2] = alloc_small(3,0);
+  Field(r[2], 0) = (value)cinfop;
+  Field(r[2], 1) = (value)infile;
+  Field(r[2], 2) = (value)jerrp;
+
+  r[3] = caml_val_jpeg_rev_markers( cinfop->marker_list );
+
+  res = alloc_small(4,0);
+  for(i=0; i<4; i++) Field(res, i) = r[i];
+
   CAMLreturn(res);
 }
 
@@ -313,6 +214,7 @@ value open_jpeg_file_for_read_start( jpegh )
 {
   CAMLparam1(jpegh);
   CAMLlocal1(res);
+  CAMLlocalN(r,3);
   struct jpeg_decompress_struct* cinfop;
   struct my_error_mgr *jerrp;
   FILE *infile;
@@ -339,6 +241,8 @@ value open_jpeg_file_for_read_start( jpegh )
   /* Step 5: Start decompressor */
 
   (void) jpeg_start_decompress(cinfop);
+  DEBUGF("jpeg_start_decompress");
+
   /* We can ignore the return value since suspension is not possible
    * with the stdio data source.
    */
@@ -353,44 +257,53 @@ value open_jpeg_file_for_read_start( jpegh )
 
   /* row_stride = cinfop->output_width * cinfop->output_components; */
 
-  { 
-    CAMLlocalN(r,3);
-    r[0] = Val_int(cinfop->output_width);
-    r[1] = Val_int(cinfop->output_height);
-    r[2] = alloc_tuple(3);
-    Field(r[2], 0) = (value)cinfop;
-    Field(r[2], 1) = (value)infile;
-    Field(r[2], 2) = (value)jerrp;
-    res = alloc_tuple(3);
-    for(i=0; i<3; i++) Field(res, i) = r[i];
-  }
-#ifdef DEBUG_JPEG
-  fprintf(stderr, "cinfop= %d infile= %d %d %d \n", cinfop, infile, cinfop->output_scanline, cinfop->output_height); 
-  fflush(stderr);
-#endif
+  // CR jfuruse: integer overflow
+  r[0] = Val_int(cinfop->output_width);
+  r[1] = Val_int(cinfop->output_height);
+  r[2] = alloc_small(3,0);
+  Field(r[2], 0) = (value)cinfop;
+  Field(r[2], 1) = (value)infile;
+  Field(r[2], 2) = (value)jerrp;
+  res = alloc_small(3,0);
+  for(i=0; i<3; i++) Field(res, i) = r[i];
+
+  DEBUGF("cinfop= %d infile= %d %d %d \n", (int)cinfop, (int)infile, cinfop->output_scanline, cinfop->output_height); 
   CAMLreturn(res);
 }
 
-value read_jpeg_scanline( jpegh, buf )
-value jpegh, buf;
+value read_jpeg_scanline( jpegh, buf, offset )
+value jpegh, offset, buf;
 {
-  CAMLparam2(jpegh,buf);
-  
+  CAMLparam3(jpegh,offset,buf);
   struct jpeg_decompress_struct *cinfop;
   JSAMPROW row[1];
 
   cinfop = (struct jpeg_decompress_struct *) Field( jpegh, 0 );
-
-  row[0] = String_val( buf );
-
-  /*
-  fprintf(stderr, "scan %d %d ...\n", cinfop->output_scanline, cinfop->output_height);
-  fflush(stderr);
-  */
+  row[0] = String_val( buf ) + Int_val( offset );
   jpeg_read_scanlines( cinfop, row, 1 );
+
   CAMLreturn(Val_unit);
 }
 
+/* no boundary checks */
+void read_jpeg_scanlines( value jpegh, value buf, value offset, value lines )
+{
+  CAMLparam4(jpegh,offset,buf,lines);
+  struct jpeg_decompress_struct *cinfop;
+  JSAMPROW row[1];
+  int clines = Int_val(lines);
+  int i;
+  row[0] = String_val(buf) + Int_val(offset);
+  cinfop = (struct jpeg_decompress_struct *) Field( jpegh, 0 );
+  // width is NOT image_width since we may have scale_denom <> 1
+  int scanline_bytes = cinfop->output_width * 3; // no padding (size 3 is fixed? )
+  for(i=0; i<clines; i++){
+    jpeg_read_scanlines( cinfop, row, 1 );
+    row[0] += scanline_bytes;
+  }
+  CAMLreturn0;
+}
+
 value close_jpeg_file_for_read( jpegh )
      value jpegh;
 {
@@ -400,84 +313,51 @@ value close_jpeg_file_for_read( jpegh )
   struct my_error_mgr *jerrp;
   FILE *infile;
 
-#ifdef DEBUG_JPEG
-  fprintf(stderr, "closing\n");
-  fflush(stderr);
-#endif
+  DEBUGF("closing\n");
 
   cinfop = (struct jpeg_decompress_struct *) Field( jpegh, 0 );
   infile = (FILE *) Field( jpegh, 1 );
   jerrp = (struct my_error_mgr *) Field( jpegh, 2 );
 
-#ifdef DEBUG_JPEG
-  fprintf(stderr, "cinfop= %d infile= %d %d %d \n", cinfop, infile, cinfop->output_scanline, cinfop->output_height); 
-#endif
+  DEBUGF("cinfop= %d infile= %d %d %d \n", (int)cinfop, (int)infile, cinfop->output_scanline, cinfop->output_height); 
 
-  if( cinfop->output_scanline >= cinfop->output_height ){ 
-#ifdef DEBUG_JPEG
-    fprintf(stderr, "finish\n");
-    fflush(stderr);
-#endif
-    jpeg_finish_decompress( cinfop );
+  if( cinfop->output_height > 0 ){ // == 0 means jpeg_start_decompress is never called
+      if( cinfop->output_scanline >= cinfop->output_height ){ 
+          DEBUGF("finish_decompress\n");
+          jpeg_finish_decompress( cinfop );
+      }
   }
-#ifdef DEBUG_JPEG
-  fprintf(stderr, "destroy\n");
-  fflush(stderr);
-#endif
+  DEBUGF("destroy\n");
   jpeg_destroy_decompress( cinfop ); 
   
   free(cinfop);
   free(jerrp);
-#ifdef DEBUG_JPEG
-  fprintf(stderr, "file close\n");
-  fflush(stderr);
-#endif
+  DEBUGF("file close\n");
   fclose(infile);
 
   CAMLreturn(Val_unit);
 }
 
-#else
+#else // HAS_JPEG
 
-value read_JPEG_file (value name) 
-{
-  CAMLparam1(name);
-  jpeg_not_supported();
-}
+#include <caml/mlvalues.h>
+#include <caml/alloc.h>
+#include <caml/memory.h>
+#include <caml/fail.h>
 
-value open_jpeg_file_for_read( name )
-     value name;
-{
-  CAMLparam1(name);
-  jpeg_not_supported();
-}
+value jpeg_set_scale_denom(){ failwith("unsupported"); }
+value open_jpeg_file_for_read(){ failwith("unsupported"); }
+value open_jpeg_file_for_read_start(){ failwith("unsupported"); }
+value read_jpeg_scanline(){ failwith("unsupported"); }
+value read_jpeg_scanlines(){ failwith("unsupported"); }
+value close_jpeg_file_for_read(){ failwith("unsupported"); }
+value open_jpeg_file_for_write_colorspace(){ failwith("unsupported"); }
+value open_jpeg_file_for_write(){ failwith("unsupported"); }
+value open_jpeg_file_for_write_cmyk(){ failwith("unsupported"); }
+value write_jpeg_scanline(){ failwith("unsupported"); }
+value close_jpeg_file_for_write(){ failwith("unsupported"); }
+value read_JPEG_file(){ failwith("unsupported"); }
+value write_JPEG_file(){ failwith("unsupported"); }
+void caml_jpeg_write_marker(){ failwith("unsupported"); }
 
-value open_jpeg_file_for_read_start( jpegh )
-     value jpegh;
-{
-  CAMLparam1(jpegh);
-  jpeg_not_supported();
-}
-
-value read_jpeg_scanline( jpegh, buf )
-     value jpegh, buf;
-{
-  CAMLparam2(jpegh,buf);
-  jpeg_not_supported();
-}
-
-value close_jpeg_file_for_read (value jpegh) 
-{
-  CAMLparam1(jpegh);
-  jpeg_not_supported();
-}
-
-value jpeg_set_scale_denom( jpegh, denom )
-     value jpegh;
-     value denom;
-{
-  CAMLparam2(jpegh,denom);
-  jpeg_not_supported();
-}
-
-#endif /* HAVE_JPEG */
+#endif // HAS_JPEG
