@@ -11,41 +11,46 @@
 /*  Distributed only by permission.                                    */
 /*                                                                     */
 /***********************************************************************/
-#include <config.h>
 
-#if HAVE_PNG
+/* $Id: Exp */
+
+#include "../config/config.h"
+
+#ifdef HAS_PNG
+
 #include <png.h>
-#endif
 
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
 #include <caml/fail.h>
 
-#if HAVE_PNG
-value write_png_file_rgb24( name, buffer, width, height )
+value write_png_file_rgb( name, buffer, width, height, with_alpha )
      value name;
      value buffer;
      value width;
      value height;
+     value with_alpha;
 {
-  CAMLparam4 ( name, buffer, width, height );
+  CAMLparam5 ( name, buffer, width, height, with_alpha );
 
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
 
   int w, h;
+  int a;
 
   w = Int_val(width);
   h = Int_val(height);
+  a = Bool_val(with_alpha);
 
   if (( fp = fopen(String_val(name), "wb")) == NULL ){
     failwith("png file open failed");
   }
 
   if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
-					NULL, NULL, NULL)) == NULL ){
+                                        NULL, NULL, NULL)) == NULL ){
     fclose(fp);
     failwith("png_create_write_struct");
   }
@@ -57,7 +62,7 @@ value write_png_file_rgb24( name, buffer, width, height )
   }
 
   /* error handling */
-  if (setjmp(png_ptr->jmpbuf)) {
+  if (setjmp(png_jmpbuf(png_ptr))) {
     /* Free all of the memory associated with the png_ptr and info_ptr */
     png_destroy_write_struct(&png_ptr, &info_ptr);
     fclose(fp);
@@ -69,21 +74,21 @@ value write_png_file_rgb24( name, buffer, width, height )
   png_init_io(png_ptr, fp);
 
   /* we use system default compression */
-  /* png_set_filter( png_ptr, 0, PNG_FILTER_NONE | 
+  /* png_set_filter( png_ptr, 0, PNG_FILTER_NONE |
      PNG_FILTER_SUB | PNG_FILTER_PAETH ); */
   /* png_set_compression...() */
 
   png_set_IHDR( png_ptr, info_ptr, w, h,
-		8 /* fixed */, 
-		PNG_COLOR_TYPE_RGB, /* fixed */
-		PNG_INTERLACE_ADAM7,
-		PNG_COMPRESSION_TYPE_DEFAULT, 
-		PNG_FILTER_TYPE_DEFAULT );
+                8 /* fixed */,
+                a ? PNG_COLOR_TYPE_RGB_ALPHA : PNG_COLOR_TYPE_RGB, /* fixed */
+                PNG_INTERLACE_ADAM7,
+                PNG_COMPRESSION_TYPE_DEFAULT,
+                PNG_FILTER_TYPE_DEFAULT );
 
   /* infos... */
 
   png_write_info(png_ptr, info_ptr);
-  
+
   {
     int rowbytes, i;
     png_bytep *row_pointers;
@@ -93,12 +98,12 @@ value write_png_file_rgb24( name, buffer, width, height )
 
     rowbytes= png_get_rowbytes(png_ptr, info_ptr);
 #if 0
-    printf("rowbytes= %d width=%d\n", rowbytes, w);    
+    printf("rowbytes= %d width=%d\n", rowbytes, w);
 #endif
     for(i=0; i< h; i++){
       row_pointers[i] = (png_bytep)(buf + rowbytes * i);
     }
-    
+
     png_write_image(png_ptr, row_pointers);
     stat_free((void*)row_pointers);
   }
@@ -115,7 +120,7 @@ void PngPalette_val( value cmap, png_colorp *pltep, int *lenp )
 {
   int i;
 
-  if( cmap == Atom(0) ){ 
+  if( cmap == Atom(0) ){
     *pltep = NULL;
     *lenp = 0;
     return;
@@ -154,7 +159,7 @@ value write_png_file_index( name, buffer, cmap, width, height )
   }
 
   if ((png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
-					NULL, NULL, NULL)) == NULL ){
+                                        NULL, NULL, NULL)) == NULL ){
     fclose(fp);
     failwith("png_create_write_struct");
   }
@@ -166,7 +171,7 @@ value write_png_file_index( name, buffer, cmap, width, height )
   }
 
   /* error handling */
-  if (setjmp(png_ptr->jmpbuf)) {
+  if (setjmp(png_jmpbuf(png_ptr))) {
     /* Free all of the memory associated with the png_ptr and info_ptr */
     png_destroy_write_struct(&png_ptr, &info_ptr);
     fclose(fp);
@@ -178,16 +183,16 @@ value write_png_file_index( name, buffer, cmap, width, height )
   png_init_io(png_ptr, fp);
 
   /* we use system default compression */
-  /* png_set_filter( png_ptr, 0, PNG_FILTER_NONE | 
+  /* png_set_filter( png_ptr, 0, PNG_FILTER_NONE |
      PNG_FILTER_SUB | PNG_FILTER_PAETH ); */
   /* png_set_compression...() */
 
   png_set_IHDR( png_ptr, info_ptr, w, h,
-		8 /* fixed */, 
-		PNG_COLOR_TYPE_PALETTE, /* fixed */
-		PNG_INTERLACE_ADAM7,
-		PNG_COMPRESSION_TYPE_DEFAULT, 
-		PNG_FILTER_TYPE_DEFAULT );
+                8 /* fixed */,
+                PNG_COLOR_TYPE_PALETTE, /* fixed */
+                PNG_INTERLACE_ADAM7,
+                PNG_COMPRESSION_TYPE_DEFAULT,
+                PNG_FILTER_TYPE_DEFAULT );
 
   {
     png_colorp palette;
@@ -207,7 +212,7 @@ value write_png_file_index( name, buffer, cmap, width, height )
   /* infos... */
 
   png_write_info(png_ptr, info_ptr);
-  
+
   {
     int rowbytes, i;
     png_bytep *row_pointers;
@@ -217,7 +222,7 @@ value write_png_file_index( name, buffer, cmap, width, height )
 
     rowbytes= png_get_rowbytes(png_ptr, info_ptr);
 #if 0
-    printf("rowbytes= %d width=%d\n", rowbytes, w);    
+    printf("rowbytes= %d width=%d\n", rowbytes, w);
 #endif
 
     if( rowbytes != w && rowbytes != w * 2 ){
@@ -229,7 +234,7 @@ value write_png_file_index( name, buffer, cmap, width, height )
     for(i=0; i< h; i++){
       row_pointers[i] = (png_bytep)(buf + rowbytes * i);
     }
-    
+
     png_write_image(png_ptr, row_pointers);
     stat_free((void*)row_pointers);
   }
@@ -242,14 +247,4 @@ value write_png_file_index( name, buffer, cmap, width, height )
   CAMLreturn(Val_unit);
 }
 
-#else
-
-#define nodef(f) value f(){failwith( "png is not supported" ); \
-  return(Val_unit);}
-
-nodef(write_png_file)
-nodef(write_png_file_index)
-nodef(write_png_file_rgb24)
-
-#endif
- 
+#endif // HAS_PNG
