@@ -10,12 +10,10 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: oTiff.ml,v 1.7 2004/09/07 07:31:07 weis Exp $ *)
+(* $Id: oTiff.ml,v 1.1 2007/01/18 10:29:57 rousse Exp $ *)
 
-open OImages;;
-
-let load name opts = OImages.make (Tiff.load name opts);;
+let load name opts = OImages.make (Tiff.load name opts)
 
 let save name opts image =
   let rgb24 = OImages.rgb24 image in
-  Tiff.save name opts rgb24#image;;
+  Tiff.save name opts rgb24#image
