@@ -12,33 +12,33 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: tiff.mli,v 1.11 2004/09/07 07:31:07 weis Exp $ *)
+(* $Id: tiff.mli,v 1.1 2007/01/18 10:29:57 rousse Exp $ *)
 
-val check_header : string -> Images.header;;
+val check_header : string -> Images.header
   (** Checks the file header *)
 
-val load : string -> Images.load_option list -> Images.t;;
+val load : string -> Images.load_option list -> Images.t
   (** Loads a tiff image. *)
 
-val save : string -> Images.save_option list -> Images.t -> unit;;
+val save : string -> Images.save_option list -> Images.t -> unit
   (** Save a full-color image in tiff format file.
      Raises [Invalid_argument] if the image is not a full-color image. *)
 
 (** Scanline based I/O functions *)
 
-type colormodel = RGB | CMYK | WHITEBLACK | BLACKWHITE;;
+type colormodel = RGB | CMYK | WHITEBLACK | BLACKWHITE
 
-type in_handle;;
+type in_handle
 
-val open_in : string -> int * int * float * colormodel * in_handle;;
-val read_scanline : in_handle -> string -> int -> unit;;
-val close_in : in_handle -> unit;;
+val open_in : string -> int * int * float * colormodel * in_handle
+val read_scanline : in_handle -> bytes -> int -> unit
+val close_in : in_handle -> unit
 
-type out_handle;;
+type out_handle
 
-val open_out : string -> int -> int -> float -> out_handle;;
-val write_scanline : out_handle -> string -> int -> unit;;
-val close_out : out_handle -> unit;;
+val open_out : string -> int -> int -> float -> out_handle
+val write_scanline : out_handle -> bytes -> int -> unit
+val close_out : out_handle -> unit
 
 
 
