@@ -11,26 +11,33 @@
 /*  Distributed only by permission.                                    */
 /*                                                                     */
 /***********************************************************************/
-#include <config.h>
+
+#include "../config/config.h"
+
+#ifdef HAS_TIFF
+
 #include <caml/mlvalues.h>
 #include <caml/alloc.h>
 #include <caml/memory.h>
 #include <caml/fail.h>
 
-#if HAVE_TIFF
-
-/* These are defined in caml/config.h */
-#define int16 int16tiff
-#define uint16 uint16tiff
-#define int32 int32tiff
-#define uint32 uint32tiff
-
-#include <tiffio.h>
-
+// This is to resolve the conflict of these int types in caml/config.h and tiff.h
+// Replace all the occurrences of (u?int[0-9]+) by $1_tiff, tiff.h's int types have
+// now different names from caml's.
 #undef int16
 #undef uint16
 #undef int32
 #undef uint32
+#undef int64
+#undef uint64
+#define int16 int16_tiff
+#define uint16 uint16_tiff
+#define int32 int32_tiff
+#define uint32 uint32_tiff
+#define int64 int64_tiff
+#define uint64 uint64_tiff
+
+#include <tiffio.h>
 
 extern value *imglib_error;
 
@@ -44,7 +51,6 @@ value open_tiff_file_for_write( value file,
   int image_height;
   double res;
   TIFF* tif;
-  int y;
 
   image_width = Int_val( width );
   image_height = Int_val( height );
@@ -87,68 +93,4 @@ value write_tiff_scanline( tiffh, buf, row )
   CAMLreturn(Val_unit);
 }
 
-/* Not supported anymore */
-#ifdef OLDCODE
-value write_tiff_file( value file,
-		       value buffer,
-		       value width,
-		       value height,
-                       value resolution )
-{
-  CAMLparam5(file,buffer,width,height,resolution);
-  char *buf;
-  char *filename; 
-  int image_width;
-  int image_height;
-  double res;
-  TIFF* tif;
-  int y;
-
-  buf = String_val ( buffer );
-  image_width = Int_val( width );
-  image_height = Int_val( height );
-  res = Double_val( resolution );
-
-  tif = TIFFOpen(String_val(file), "w");
-  if( tif ){
-    /* needs */
-    /* FillOrder */
-    
-    TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, (uint32) image_width);
-    TIFFSetField(tif, TIFFTAG_IMAGELENGTH, (uint32) image_height);
-    TIFFSetField(tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
-    TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 3);
-    TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
-    TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
-    TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_LZW); /* LZW */
-    TIFFSetField(tif, TIFFTAG_PREDICTOR, 2); /* ??? */
-    TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
-    TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, image_height); /* ??? */
-    TIFFSetField(tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_INCH);
-    TIFFSetField(tif, TIFFTAG_XRESOLUTION, res);
-    TIFFSetField(tif, TIFFTAG_YRESOLUTION, res);
-    for(y=0; y<image_height; y++){
-      TIFFWriteScanline(tif, buf + y * image_width * 3, y, 0);
-    }
-    
-    TIFFClose(tif);
-  } else {
-    failwith("failed to open tiff file to write");
-  }
-  CAMLreturn(Val_unit);
-}
-#endif /* OLDCODE */
-
-#else
-
-#define tiff_not_supported() \
-  failwith( "tiff is not supported" ); \
-  return(Val_unit)
-
-#define nodef(f) value f(){tiff_not_supported();}
-
-nodef(write_tiff_file)
-nodef(open_tiff_file_for_write)
-nodef(write_tiff_scanline)
-
-#endif
+#endif // HAS_TIFF
